@@ -432,6 +432,16 @@ class CommandText(AggregateToken):
         in case of error."""
         self.modify("", begin=loc, end=loc+count)
 
+    def modify(self, txt="", begin=0, end=None):
+        # Handle the one case where super.modify() may leave an Arg at the
+        # beginning instead of a Prog, and reparse the whole thing.
+        ret = super(CommandText, self).modify(txt, begin, end)
+        if len(self.tokens) > 0 and isinstance(self.tokens[0], Arg):
+            rawtext = self.__str__()
+            self.tokens = []
+            return self.modify(rawtext)
+        return ret
+
     def create_token(self, txt, loc):
         if re.match("\s", txt):
             return ArgSeparator()
@@ -477,7 +487,7 @@ class ArgOrProg(AggregateToken):
 
     def __init__(self):
         super(ArgOrProg, self).__init__()
-        self.escaper = re.compile("(["+ re.escape(commencers + "*?[]") +"])")
+        self.escaper = re.compile("(["+ re.escape(commencers + "*?[] \t") +"])")
 
     def create_token(self, txt, loc):
         if re.match("\s", txt):
@@ -635,35 +645,32 @@ class Prog(ArgOrProg):
                 return [expnd]
             else:
                 return ['']
-        elif expnd.startswith(config.builtin_commencer):
-            # Explicitly stated builtin
-            if (expnd[len(config.builtin_commencer):] in jobs.loaded_builtins):
+
+        # Check builtins and aliases if they weren't suppressed.
+        if not expnd.startswith(config.builtin_suppressor):
+            if expnd in aliases.manager and self.do_aliases:
+                return aliases.manager.get_parsed_value(expnd).expand()
+
+            if expnd in jobs.loaded_builtins:
                 return [expnd]
-            else:
-                return ['']
-
-        elif expnd in aliases.manager and self.do_aliases:
-            return aliases.manager.get_parsed_value(expnd).expand()
-
-        elif expnd in jobs.loaded_builtins:
-            # Implicit builtin
-            return ['+' + expnd]
-
         else:
-            # Treat as a relative specification, so search the path.
-            expnd_mtch = []
+            # Remove the builtin_suppressor
+            expnd = expnd[len(config.builtin_suppressor):]
 
-            for pathe in os.environ["PATH"].split(os.path.pathsep):
-                ea = os.path.join(pathe, expnd)
-                if os.path.exists(ea):
-                    expnd_mtch.append(ea)
+        # Relative specification, so search the path.
+        expnd_mtch = []
 
-            if len(expnd_mtch) == 0:
-                return ['']
-            if len(expnd_mtch) > 1:
-                hsh.display_obj.show_alert('Multiple commands found on path')
+        for pathe in os.environ["PATH"].split(os.path.pathsep):
+            ea = os.path.join(pathe, expnd)
+            if os.path.exists(ea):
+                expnd_mtch.append(ea)
 
-            return expnd_mtch[0:1]
+        if len(expnd_mtch) == 0:
+            return ['']
+        if len(expnd_mtch) > 1:
+            hsh.display_obj.show_alert('Multiple commands found on path')
+
+        return expnd_mtch[0:1]
 
     def expand_directives(self):
         """Only aliases can include directives."""
@@ -690,39 +697,37 @@ class Prog(ArgOrProg):
 
         comps = []
 
-        if expnd.startswith(config.builtin_commencer):
-            # expand from the list of builtins
-            bimatch = expnd[len(config.builtin_commencer):]
-            for biname in jobs.loaded_builtins.keys():
-                if biname.startswith(bimatch):
-                    comps.append(biname[len(bimatch):])
+        use_builtins = True
+        if expnd.startswith(config.builtin_suppressor):
+            use_builtins = False
+            expnd = expnd[len(config.builtin_suppressor):]
 
+        # Form a list of filenames on which to do glob expansion
+        if Prog.expl_cmd_re.match(expnd):
+            # Explicit specification so don't use $PATH
+            srchpaths = [expnd]
         else:
-            # Form a list of filenames on which to do glob expansion
-            if Prog.expl_cmd_re.match(expnd):
-                # Explicit specification so don't use $PATH
-                srchpaths = [expnd]
-            else:
-                srchpaths = map(lambda x: os.path.join(x, expnd),
-                                os.environ["PATH"].split(os.path.pathsep))
+            srchpaths = map(lambda x: os.path.join(x, expnd),
+                            os.environ["PATH"].split(os.path.pathsep))
 
-            for sp in srchpaths:
-                gres = glob.glob(sp + "*")
-                # Append / to directories
-                gres = map(lambda x: os.path.isdir(x) and x+os.sep or x, gres)
-                # Trim the common bit off.
-                gres = map(lambda x: x[len(sp):], gres)
-                comps.extend(gres)
+        for sp in srchpaths:
+            gres = glob.glob(sp + "*")
+            # Append / to directories
+            gres = map(lambda x: os.path.isdir(x) and x+os.sep or x, gres)
+            # Trim the common bit off.
+            gres = map(lambda x: x[len(sp):], gres)
+            comps.extend(gres)
 
-            # Add any matching builtins
+        # Add any matching builtins
+        if use_builtins:
             for biname in jobs.loaded_builtins.keys():
                 if biname.startswith(expnd):
                     comps.append(biname[len(expnd):])
 
-            # Add any matching aliases
-            for alname in aliases.manager:
-                if alname.startswith(expnd) and self.do_aliases:
-                    comps.append(alname[len(expnd):])
+        # Add any matching aliases
+        for alname in aliases.manager:
+            if alname.startswith(expnd) and self.do_aliases:
+                comps.append(alname[len(expnd):])
 
         # Sort and remove duplicates.
         comps.sort()
@@ -743,7 +748,8 @@ class HashDirective(LeafToken):
     commencer_ch = "#"
     # Grouped into mutually-exclusive sets.
     directives_g = [["exclusive", "piped", "pty"],
-                    ["hide", "hideonsuccess", "show"]]
+                    ["changeview", "dontchangeview"],
+                    ]
     directives = reduce(lambda x,y: x+y, directives_g)
 
     @staticmethod
@@ -995,7 +1001,7 @@ class JobReference(BracedToken):
 
     def _expand_feature(self, job, feature):
         if feature == "p":
-            return [str(job.pid) or ""]
+            return [str(job.pid or "")]
         if feature[0:1] == "f":
             fname = self._expand_feature_filetype(feature[1:2])
             if fname is not None:
