@@ -89,10 +89,10 @@ def run():
         signal.signal(signal.SIGTSTP, signal.SIG_IGN)
         signal.signal(signal.SIGTTIN, signal.SIG_IGN)
         signal.signal(signal.SIGTTOU, signal.SIG_IGN)
-        signal.signal(signal.SIGCHLD, signal.SIG_IGN)
 
-        # Put process into a new group.
-        os.setpgid(os.getpid(), os.getpid())
+        # If this process isn't the group leader, put it in a new group.
+        if os.getpid() != os.getsid(0):
+            os.setpgid(os.getpid(), os.getpid())
         # Put the new group into the foreground.
         os.tcsetpgrp(sys.stdin.fileno(), os.getpgrp())
         # Remember terminal settings.
@@ -136,9 +136,9 @@ class CursesDisplay(hsh.display.HshDisplay, hsh.jobs.JobManagerListener):
 
         self.initialize_faces()
 
-        # A dict mapping job ids to job display objects.
-        # Use get_job_display() to get a display.
-        self.job_display_index = dict()
+        # A dict mapping job ids to job view objects.
+        # Use get_job_view() to get a view.
+        self.job_view_index = dict()
 
         self.views = {}
         self.views["JobView"] = None
@@ -194,10 +194,10 @@ class CursesDisplay(hsh.display.HshDisplay, hsh.jobs.JobManagerListener):
             cnum += 1
         logging.debug(str(self.faces))
 
-    def get_job_display(self, job):
-        if job.jid not in self.job_display_index:
-            self.job_display_index[job.jid] = JobView(self, job)
-        return self.job_display_index[job.jid]
+    def get_job_view(self, job):
+        if job.jid not in self.job_view_index:
+            self.job_view_index[job.jid] = JobView(self, job)
+        return self.job_view_index[job.jid]
 
     def give_focus(self, displayable):
         if self.focus == displayable:
@@ -324,12 +324,14 @@ class CursesDisplay(hsh.display.HshDisplay, hsh.jobs.JobManagerListener):
         if cd != self.main:
             self.main = cd
 
-    def display_job(self, job, show=True, focus=False):
+    def display_job(self, job, show=None, focus=False):
         """Set the job being displayed, if show is False, then the job
         displayable is updated, but the job is not shown.  job may be a
         hsh.jobs.Job object, a JobView object, -1 to mean move back a
         job, +1 to mean move forward a job, or 0 to mean the current one is
         gone, and a new one needs to be chosen."""
+        if show == None:
+            show = self.views["JobView"] == self.main
         if type(job) == int:
             jv = self.views["JobView"]
             sl = self.views["SessionList"]
@@ -337,11 +339,11 @@ class CursesDisplay(hsh.display.HshDisplay, hsh.jobs.JobManagerListener):
                 job = None
             if job == -1:
                 job = sl.get_predecessor(jv.job)
-                if job is None: # Don't fall of the end
+                if job is None: # Don't fall off the end
                     job = jv
             elif job == 1:
                 job = sl.get_successor(jv.job)
-                if job is None: # Don't fall of the end
+                if job is None: # Don't fall off the end
                     job = jv
             elif job == 0:
                 job = sl.get_successor(jv.job)
@@ -349,12 +351,20 @@ class CursesDisplay(hsh.display.HshDisplay, hsh.jobs.JobManagerListener):
                     job = sl.get_predecessor(jv.job)
 
         if isinstance(job, hsh.jobs.Job):
-            job = self.get_job_display(job)
+            job = self.get_job_view(job)
 
         # job is now a JobView or None
         self.views["JobView"] = job
         if show:
             self.set_main(job, refocus=focus)
+            jobid = job and job.job or None
+            for view in self.views.values():
+                if view is not None:
+                    view.set_visible_job(jobid)
+
+    def display_sessionlist(self):
+        "Show the session list when input has created a new job."
+        self.set_main(self.views["SessionList"])
 
     def relinquish_terminal(self, job):
         # Deinitialize curses so that a subprocess can take control of the
@@ -403,7 +413,7 @@ class CursesDisplay(hsh.display.HshDisplay, hsh.jobs.JobManagerListener):
     def on_remove_job(self, job):
         """If the removed job is the currently displayed one, update."""
         if self.views["JobView"] and self.views["JobView"].job == job:
-            self.display_job(0, show = self.views["JobView"] == self.main)
+            self.display_job(0)
 
     ######################################################################
     # HshDisplay Interface
@@ -423,6 +433,13 @@ class CursesDisplay(hsh.display.HshDisplay, hsh.jobs.JobManagerListener):
         else:
             self.show_alert("No view with that name")
 
+    def get_view(self, view_name):
+        """Return a view object for the named view."""
+        if view_name in self.views:
+            return self.views[view_name]
+        else:
+            self.show_alert("No view with that name")
+
     def list_views(self):
         """Return a list of view names."""
         return self.views.keys()
