@@ -29,6 +29,8 @@ keys_default = {
     (curses.KEY_UP, False):            "move_up",
     (curses.KEY_NPAGE, False):         "page_down",
     (curses.KEY_PPAGE, False):         "page_up",
+    (curses.KEY_HOME, False):          "move_top",
+    (curses.KEY_END, False):           "move_bottom",
     (curses.KEY_BACKSPACE, False):     "delete_left",
     (curses.KEY_DC, False):            "delete_right",
     (curses.ascii.SOH, False):         "move_start",   # ^A
@@ -66,13 +68,23 @@ keys_InputView = {
     (curses.ascii.ETX, False):         "cancel",           # ^C
     (curses.ascii.TAB, False):         "tab_complete",     # ^I
     (curses.ascii.NL, False):          "do_command",
-    (curses.KEY_NPAGE, False):         "main_page_down",
-    (curses.KEY_PPAGE, False):         "main_page_up",
+    (curses.KEY_NPAGE, False):         "main_handle",
+    (curses.KEY_PPAGE, False):         "main_handle",
+    (curses.KEY_HOME, False):          "main_handle",
+    (curses.KEY_END, False):           "main_handle",
+    # These should use the predicate below . . .
+    (ord('d'), True):                  "main_handle",
+    (ord('h'), True):                  "main_handle",
+    (ord('n'), True):                  "main_handle",
+    (ord('p'), True):                  "main_handle",
+    (ord('r'), True):                  "main_handle",
     (ord('q'), True):                  "dump_parse_tree",
 }
 
-preds_InputView = {
-}
+preds_InputView = [
+    # This does nothing useful because default keybindings are checked first.
+    (lambda cl,ki: ki.meta,            "main_handle"),
+]
 
 keys_JobView = {
     (ord('i'), True):                  "cycle_input",
@@ -86,6 +98,9 @@ keys_SessionList = {
     (curses.ascii.EOT, False):         "delete_job",  # ^D
     (curses.KEY_BACKSPACE, False):     "delete_job",
     (curses.KEY_DC, False):            "delete_job",
+    (curses.KEY_HOME, False):          "move_job_top",
+    (curses.KEY_END, False):           "move_job_bottom",
+    (ord('h'), True):                  "hide_output",
 }
 
 keys_CommandHistory = {
