@@ -23,6 +23,7 @@ import curses
 import logging
 
 import hsh
+import hsh.config
 import hsh.content
 
 from hsh.exceptions import *
@@ -68,7 +69,7 @@ class View(object):
 
     @staticmethod
     def format_lines(display, lines, height, width, dpos=[0,0], wrap=True,
-                     face=None, wrapface=None, trim=0):
+                     face=None, wrapface=None, trim=0, tail=False):
         """Format the provided lines into lines of appropriate size for display.
           lines is a TextContent or a list of NL terminated strings
           wrap  is a boolean, if True, lines should be wrapped
@@ -83,8 +84,13 @@ class View(object):
           face is the face to use for output lines.  If None and lines is a
                TextContent, then the face from lines will be used, otherwise
                no face will be set.
+          wrapface is the face used for line wrap indicators.
           trim is the number of characters to trim from the left of each line.
 
+          tail is a boolean defaulting to False, if True formatting will start
+               at the end of lines, working up, and dpos's will count lines
+               from the bottom, and characters from the right.
+
         A TextContent object is returned, preserving any formatting regions of
         the original source.  Additionally, each TextLine within the content
         has its "line_number" variable set to the number of the line from which
@@ -101,78 +107,128 @@ class View(object):
         if not wrapface:
             wrapface=display.faces["wrapmark"]
 
-        def wrap_line(lc):
-            rg = content_fmt.append_region('\\\n')
-            rg.face = wrapface
-            rg = content_fmt.append_region('/')
-            rg.face = wrapface
-            rg.get_line().line_number = lc
+        if not tail:
+            # Start formatting from the first line and append
+            lnum = dpos[0]
+            while len(content_fmt) < height and lnum < len(lines):
+                line = lines[lnum]
+                ltrim = (lnum == dpos[0]) and max(trim, dpos[1]) or trim
 
-        def trunc_line():
-            rg = content_fmt.append_region('_')
-            rg.face = wrapface
+                line_fmt = View._format_line(line, wrap_width, wrap, face,
+                                             wrapface, ltrim, lnum)
 
-        def start_line(lc):
-            if trim > 0:
-                rg = content_fmt.append_region('-')
+                content_fmt.extend(line_fmt)
+                lnum += 1
+
+        else:
+            # Start formatting from the last line and prepend
+            lnum = len(lines) - dpos[0] - 1
+            while len(content_fmt) < height and lnum >= 0:
+                line = lines[lnum]
+                if dpos[1] > 0 and lnum == (len(lines) - dpos[0]):
+                    line = line[0:len(line) - dpos[1]]
+                line_fmt = View._format_line(line, wrap_width, wrap, face,
+                                             wrapface, trim, lnum)
+
+                content_fmt[:0] = line_fmt
+                lnum -= 1
+
+        return content_fmt
+
+    @staticmethod
+    def _format_line_wrap(content_fmt, wrapface, lc):
+        "Internal function for _format_line()"
+        rg = content_fmt.append_region('\\\n')
+        rg.face = wrapface
+        rg = content_fmt.append_region('/')
+        rg.face = wrapface
+        rg.get_line().line_number = lc
+
+    @staticmethod
+    def _format_line_trunc(content_fmt, wrapface):
+        "Internal function for _format_line()"
+        rg = content_fmt.append_region('_')
+        rg.face = wrapface
+
+    @staticmethod
+    def _format_line_start(content_fmt, wrapface, lc, trim):
+        "Internal function for _format_line()"
+        if trim > 0:
+            rg = content_fmt.append_region('-')
+        else:
+            rg = content_fmt.append_region(' ')
+        rg.face = wrapface
+        rg.get_line().line_number = lc
+
+    @staticmethod
+    def _format_line_end(content_fmt):
+        "Internal function for _format_line()"
+        rg = content_fmt.append_region('\n')
+
+    @staticmethod
+    def _format_line(line, wrap_width, wrap,
+                     face, wrapface, trim, lnum):
+        """Internal function used by format_lines() to format a single line.
+
+        line - A single line to format
+        wrap_width - maximum length of lines, excluding wrap characters.
+        wrap - Boolean if line should wrap or truncate
+        face - may be None, to set text face
+        wrapface - face to use for special wrap characters
+        trim - number of characters to trim from the beginning of the line
+        lnum - Line number to be attached to resulting text content.
+
+        Returns a TextContent object, possibly with multiple lines.
+        """
+
+        content_fmt = hsh.content.TextContent()
+
+        if not isinstance(line, hsh.content.TextLine):
+            line = hsh.content.TextLine(str(line).strip('\n') + '\n')
+
+        # Traverse the line, replacing any tab characters with spaces,
+        # and removing any carriage returns.
+        tw = hsh.config.tab_width
+        i = 0
+        while i < len(line):
+            if line[i] == '\r':
+                line[i:i+1] = ''
+            if line[i] == '\t':
+                sc = (i / tw + 1) * tw - i
+                line[i:i+1] = " " * sc
+                i += sc
             else:
-                rg = content_fmt.append_region(' ')
-            rg.face = wrapface
-            rg.get_line().line_number = lc
+                i += 1
 
-        def end_line():
-            rg = content_fmt.append_region('\n')
+        View._format_line_start(content_fmt, wrapface, lnum, trim)
+        for region in line.regions:
+            if len(region) <= trim:
+                trim -= len(region)
+                continue
 
-        loff = dpos[1]  # Drop this much from the first line.
-        lc = dpos[0] - 1
-        while lc + 1 < len(lines):
-            lc += 1
-            line = lines[lc]
-            lt = max(trim - loff, 0)
+            rg = content_fmt.append_region(region)
+            if trim > 0:
+                rg[0:trim] = []
+                trim = 0
 
-            # Break up the lines, but preserve region info
-            if not isinstance(line, hsh.content.TextLine):
-                line = hsh.content.TextLine(str(line).strip('\n') + '\n')
+            if face:
+                rg.face = face
+            if content_fmt[-1][-1] == '\n':
+                content_fmt[-1][-1:] = []
+            while len(content_fmt[-1]) > wrap_width:
+                spill = "".join(content_fmt[-1][wrap_width:])
+                content_fmt[-1][wrap_width:] = []
+                if wrap:
+                    View._format_line_wrap(content_fmt, wrapface, lnum)
+                    rg = content_fmt.append_region(spill)
+                    if face:
+                        rg.face = face
+                    rg.set_attrs(region)
+                else:
+                    View._format_line_trunc(content_fmt, wrapface)
+                    break
 
-            start_line(lc)
-            for region in line.regions:
-                if len(region) <= loff:
-                    loff -= len(region)
-                    continue
-                if len(region) <= lt:
-                    lt -= len(region)
-                    continue
-
-                rg = content_fmt.append_region(region)
-                if loff > 0:
-                    rg[0:loff] = []
-                    loff = 0
-                if lt > 0:
-                    rg[0:lt] = []
-                    lt = 0
-
-                if face:
-                    rg.face = face
-                if content_fmt[-1][-1] == '\n':
-                    content_fmt[-1][-1:] = []
-                while len(content_fmt[-1]) > wrap_width:
-                    spill = "".join(content_fmt[-1][wrap_width:])
-                    content_fmt[-1][wrap_width:] = []
-                    if wrap:
-                        wrap_line(lc)
-                        rg = content_fmt.append_region(spill)
-                        if face:
-                            rg.face = face
-                        rg.set_attrs(region)
-                    else:
-                        trunc_line()
-                        break
-
-            end_line()
-            loff = 0
-            # no point in formatting content that won't fit on the screen
-            if len(content_fmt) >= height:
-                break
+        View._format_line_end(content_fmt)
 
         return content_fmt
 
@@ -303,6 +359,11 @@ class View(object):
             self.set_dirty()
         self.has_focus = has_focus
 
+    def set_visible_job(self, job):
+        # Called when the main visible job has changed, in case the view wants
+        # to update internal structures to acknowledge this.
+        pass
+
     def header_info(self):
         """Return a dict containing details to include in the header."""
         return {}
@@ -328,10 +389,11 @@ class View(object):
         return
 
     def draw_header(self, win):
-        """Draw the header into the provided window.  Return the unused portion
-        of the window.  The header is arbitrarily limited to 5 lines, a
-        restriction which should be lifted by providing a way to switch between
-        header wrap and trim displays."""
+        """Draw the header into the provided window.
+
+        The header is arbitrarily limited to 5 lines, a restriction which
+        should be lifted by providing a way to switch between header wrap and
+        trim displays."""
         (height, width) = win.getmaxyx()
 
         if len(self.header_fmt) == 0:
@@ -395,9 +457,11 @@ class View(object):
 
     def draw(self, win, force_redraw):
         """This default implementation of draw() is purely illustrative, and
-        only draws the header.  Subclasses will certainly override it.  The
-        method returns a boolean, True meaning something was drawn, used by the
-        main drawing loop to decide whether it's appropriate to pause."""
+        only draws the header.  Subclasses will certainly override it.
+
+        A boolean is returned, True meaning something was drawn or the cursor
+        position has changed.  This result is used by the main drawing loop to
+        decide whether to draw the cursor, or take a nap."""
         # Unfortunately this code will be duplicated in other draw() methods,
         # because the difficulty of sharing it is too great.
         if force_redraw:
@@ -471,6 +535,12 @@ class View(object):
     def move_end(self, ki):
         pass
 
+    def move_top(self, ki):
+        pass
+
+    def move_bottom(self, ki):
+        pass
+
     def page_up(self, ki):
         pass
 
