@@ -125,17 +125,18 @@ class JobManager(object):
     def create_job(self, cmd, cmdline, directives, state):
         """Create a new job from a command and parameters, assign it an id and
         return it.
-        cmd: A list of args, the first being the job to execute.
+        cmd: A list of strings: the job to execute followed by its args.
         cmdline: The string entered by the user, as a reference, but not used
                  in job creation.
         state: A string with the job's initial state
         directives: A list of directives for how to start the job."""
-        if cmd[0].startswith(config.builtin_commencer):
-            biname = cmd[0][len(config.builtin_commencer):]
-            jobcls = loaded_builtins[biname]
-            job = jobcls.create_job(cmd, cmdline, directives, state)
+        if cmd[0] in loaded_builtins:
+            jobcls = loaded_builtins[cmd[0]]
         else:
-            job = ExternalJob.create_job(cmd, cmdline, directives, state)
+            jobcls = ExternalJob
+
+        job = jobcls.create_job(cmd, cmdline, directives, state)
+
         # Extend the all_jobs array to include the newly created job.
         for i in range(len(self.all_jobs), job.jid):
             self.all_jobs.append(os.path.join(config.jobs_dir, str(i)))
@@ -304,13 +305,8 @@ class JobOutputThread(threading.Thread):
                         break
                     else:
                         try:
-                            # XXX I'd like to use job.ps.poll() here, but it
-                            # fails, apparently because I changed the process
-                            # group earlier.
-                            (pid, ps_code) = os.waitpid(self.job.ps.pid,
-                                                        os.WNOHANG)
-                            if pid == 0:
-                                ps_code = None
+                            ps_code = self.job.ps.poll()
+                            if ps_code is None:
                                 time.sleep(0.1)
                         except OSError, ose:
                             # The child may have gone away
@@ -526,7 +522,6 @@ def new_pgrp():
     signal.signal(signal.SIGTSTP, signal.SIG_DFL)
     signal.signal(signal.SIGTTIN, signal.SIG_DFL)
     signal.signal(signal.SIGTTOU, signal.SIG_DFL)
-    signal.signal(signal.SIGCHLD, signal.SIG_DFL)
     return True
 
 class ExternalJob(Job):
@@ -714,7 +709,7 @@ class BuiltInJob(Job):
 
     def __init__(self):
         super(BuiltInJob, self).__init__()
-        self.directives_default = ["hide"]
+        self.directives_default = ["dontchangeview"]
 
     def start(self):
         super(BuiltInJob, self).start()
