@@ -216,14 +216,14 @@ class CommandTester(unittest.TestCase):
         # Appears in command directives.
         self.assertEqual(self.ct.expand_directives(), ['pty'])
         # Multiple directives work, by defining a new directive for the test.
-        hsh.command.HashDirective.directives.append('#foo_test')
+        hsh.command.HashDirective.directives_g.append('#foo_test')
         self.ct.insert(2, " ##foo_test")
         self.assertEqual(self.ct.expand(), ['/bin/ls'])
-        self.assertEqual(self.ct.expand_directives(), ['#foo_test', 'pty'])
+        self.assertEqual(self.ct.expand_directives(), ['pty', '#foo_test'])
         # Does not appear in middle of token.
         self.ct.insert(2, " ''#exclusive")
         self.assertEqual(self.ct.expand(), ['/bin/ls', '#exclusive'])
-        self.assertEqual(self.ct.expand_directives(), ['#foo_test', 'pty'])
+        self.assertEqual(self.ct.expand_directives(), ['pty', '#foo_test'])
 
 class GlobTest(unittest.TestCase):
 
